﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "ABMPageLocoControl"
	' will hold the unique browsers window id
	Private ABMPageId As String = ""
	' your own variables
	' ABM Components
	Private labelLocoDirection As ABMLabel
	Private labelLocoSpeed As ABMLabel	'Set as class global to be able to update with the speed set
	Private labelLocoObstacleDistance As ABMLabel
	' Helpers with inital values
	Private myToastId As Int
	Private LocoDirection As String ="BRK"
	Private LocoSpeed As String = "70"
	Private LocoObstacleDistance As String = "4"
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage		
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)	
	Log("Connected")
		
	ws = WebSocket1		
	
	ABMPageId = ABM.GetPageID(page, Name,ws)
	
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	'----------------------START MODIFICATION 4.00-------------------------------
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	'----------------------END MODIFICATION 4.00-------------------------------
	
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If		
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)		
	If page.ComesFromPageCache Then
    	' when we have a page that is cached it doesn't matter if it comes or not from a new connection we serve the cached version.
		Log("Comes from cache")		
    	page.Refresh		
    	page.FinishedLoading		
	Else
    	If page.WebsocketReconnected Then
			Log("Websocket reconnected")
        	' when we have a client that doesn't have the page in cache and it's websocket reconnected and also it's session is new - basically when the client had internet problems and it's session (and also cache) expired before he reconnected so the user has content in the browser but we don't have any on the server. So we need to reload the page.
        	' when a client that doesn't have the page in cache and it's websocket reconnected but it's session is not new - when the client had internet problems and when he reconnected it's session was valid but he had no cache for this page we need to reload the page as the user browser has content, reconnected but we have no content in cache
        	ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
    	Else
        	' when the client did not reconnected it doesn't matter if the session was new or not because this is the websockets first connection so no dynamic content in the browser ... we are going to serve the dynamic content...
        	Log("Websocket first connection")
			page.Prepare
			ConnectPage			
    	End If
	End If
	Log(ABMPageId)		
End Sub

Private Sub WebSocket_Disconnected
	Log("Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	
	If eventName = "beforeunload" Then
		Log("preparing for url refresh")
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		Return
	End If
	Dim caller As Object = page.GetEventHandler(Me, eventName)
	If caller = Me Then
		If SubExists(Me, eventName) Then
			Params.Remove("eventname")
			Params.Remove("eventparams")
			' BEGIN NEW DRAGDROP
			If eventName = "page_dropped" Then
				page.ProcessDroppedEvent(Params)
			End If
			' END NEW DRAGDROP
			Select Case Params.Size
				Case 0
					CallSub(Me, eventName)
				Case 1
					CallSub2(Me, eventName, Params.Get(eventParams(0)))
				Case 2
					If Params.get(eventParams(0)) = "abmistable" Then
						Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
						CallSub2(Me, eventName, PassedTables)
					Else
						CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
					End If
				Case Else
					' cannot be called directly, to many param
					CallSub2(Me, eventName, Params)
			End Select
		End If
	Else
		CallSubDelayed2(caller, "ParseEvent", Params) 'ignore
	End If
End Sub

public Sub BuildTheme()
	' start with the base theme defined in ABMShared
	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)

	theme.AddInputTheme("inputauto")
	theme.Input("inputauto").AutoCompleteZDepth = ABM.ZDEPTH_1

	' Theme Cell alignment left, center, right
	' To use VerticalAlign set the fixedheight of a cell, i.e. page.Cell(3,2).SetFixedHeight(100, False)
	theme.AddCellTheme("alignleft")
	theme.Cell("alignleft").Align = ABM.CELL_ALIGN_LEFT
	theme.Cell("alignleft").VerticalAlign = True
	theme.AddCellTheme("aligncenter")
	theme.Cell("aligncenter").Align = ABM.CELL_ALIGN_CENTER
	theme.Cell("aligncenter").VerticalAlign = True
	theme.AddCellTheme("alignright")
	theme.Cell("alignright").Align = ABM.CELL_ALIGN_RIGHT
	theme.Cell("alignright").VerticalAlign = True
End Sub

public Sub BuildPage()
	' initialize the theme
	BuildTheme
	
	' initialize this page using our theme
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = "LocoControl"
	page.PageDescription = "Remote Control a battery powered locomotive running on Lego Duplo Train Tracks"
	page.PageKeywords = ""
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY

	' Show the connection indicator at the top right of the page		
	page.ShowConnectedIndicator = True
				
	' Adding a navigation bar (MUST)
	' NOTE: The sidebar is not used - see ABMShared Sub BuildNavigationBar
	ABMShared.BuildNavigationBar(page, page.PageTitle,"", "", "", "")

'	ABMShared.BuildNavigationBar(page, page.PageTitle,"../images/logo.png", "", "", "")
			
	' create the page grid
	' EXAMPLE = page.AddRowsM(5,True,0,0, "").AddCells12MP(1,0,0,0,0,"")

	'R1 1x6 + 1x6 = [Label Direction][Label MQTT Connection]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"alignleft").AddCellsOS(1,0,0,0,6,6,6,"alignright")

	'R2 1x6 + 1x6 = [Button BCK][Button FWD]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"aligncenter").AddCellsOS(1,0,0,0,6,6,6,"aligncenter")

	'R3 1x6 + 1x6 = [Button BRK][Button Buzzer]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,6,6,6,"aligncenter").AddCellsOS(1,0,0,0,6,6,6,"aligncenter")

	'R4 1x4 + 1x8 = [Label Speed][Slider Speed]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"alignleft").AddCellsOS(1,0,0,0,8,8,8,"alignright")

	'R5 1x4 + 1x8 = [Label Headlight][Switch Headlight]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"alignleft").AddCellsOS(1,0,0,0,8,8,8,"alignleft")

	'R6 1x4 + 1x8 = [Label Obstacle Distance][Slider Obstacle Detection Distance]
	page.AddRows(1,True,"").AddCellsOS(1,0,0,0,4,4,4,"alignleft").AddCellsOS(1,0,0,0,8,8,8,"alignright")
	
	page.BuildGrid 'IMPORTANT once you loaded the complete grid AND before you start adding components
	
	' Add the components
	' R1
'	Dim labelLocoDirection As ABMLabel
	labelLocoDirection.Initialize(page,"labelLocoDirection",LocoDirection, ABM.SIZE_PARAGRAPH, True, "bluegrey")
	page.Cell(1,1).AddComponent(labelLocoDirection)
	page.Cell(1,1).SetFixedHeight(50, False)

	Dim labelMQTTConnected As ABMLabel
	If Main.mqttConnected Then
		labelMQTTConnected.Initialize(page,"labelMQTTConnected",$"Remote Connected"$, ABM.SIZE_SMALL, True, "bluegrey")
	Else
		labelMQTTConnected.Initialize(page,"labelMQTTConnected",$"NOT Remote Connected"$, ABM.SIZE_SMALL, True, "bluegrey")
	End If
	page.Cell(1,2).AddComponent(labelMQTTConnected)
	page.Cell(1,2).SetFixedHeight(50, False)
	' R2
	Dim btnLocoReverse As ABMButton
	btnLocoReverse.InitializeRaised(page, "btnLocoReverse", "", "", "Reverse", "themebuttonorange")
	btnLocoReverse.UseFullCellWidth = True
	page.Cell(2,1).PaddingRight = "10"
	page.Cell(2,1).AddComponent(btnLocoReverse)
	Dim btnLocoForwards As ABMButton
	btnLocoForwards.InitializeRaised(page, "btnLocoForwards", "", "", "Forwards", "themebuttonblue")
	btnLocoForwards.UseFullCellWidth = True
	page.Cell(2,2).Paddingleft = "10"
	page.Cell(2,2).AddComponent(btnLocoForwards)
	' R3
	Dim btnLocoBreak As ABMButton
	btnLocoBreak.InitializeRaised(page, "btnLocoBreak", "", "", "Break", "themebuttonred")
	btnLocoBreak.UseFullCellWidth = True
	page.Cell(3,1).PaddingRight = "10"
	page.Cell(3,1).AddComponent(btnLocoBreak)
	Dim btnLocoBuzzer As ABMButton
	btnLocoBuzzer.InitializeRaised(page, "btnLocoBuzzer", "", "", "Buzzer", "themebuttonyellow")
	btnLocoBuzzer.UseFullCellWidth = True
	page.Cell(3,2).Paddingleft = "10"
	page.Cell(3,2).AddComponent(btnLocoBuzzer)
	' R4
'	Dim labelLocoSpeedLabel As ABMLabel	'Set as class global to be able to update with the speed set
	labelLocoSpeed.Initialize(page,"labelLocoSpeed",$"Speed set:{BR}${LocoSpeed} km/h"$, ABM.SIZE_PARAGRAPH, True, "bluegrey")
	page.Cell(4,1).AddComponent(labelLocoSpeed)
	page.Cell(4,1).SetFixedHeight(50, False)
	Dim sliderLocoSpeed As ABMSlider
	sliderLocoSpeed.Initialize(page, "sliderLocoSpeed", LocoSpeed, 0, 100, 1, "themesliderlocospeed")
	sliderLocoSpeed.HandleToolTip = "wNumb({ decimals: 0 })"
	sliderLocoSpeed.Connect = ABM.SLIDER_CONNECT_UPPER
	page.Cell(4,2).AddComponent(sliderLocoSpeed)
	page.Cell(4,2).SetFixedHeight(50, False)
	' R5
	Dim labelLocoHeadlight As ABMLabel	'Set as class global to be able to update with the speed set
	labelLocoHeadlight.Initialize(page,"labelLocoHeadlight",$"Headlight:"$, ABM.SIZE_PARAGRAPH, True, "bluegrey")
	page.Cell(5,1).AddComponent(labelLocoHeadlight)
	page.Cell(5,1).SetFixedHeight(50, False)
	Dim switchLocoHeadlight As ABMSwitch
	switchLocoHeadlight.Initialize(page, "switchLocoHeadlight", "ON", "OFF", False, "themeswitchlocoheadlight")
	page.Cell(5,2).AddComponent(switchLocoHeadlight)
	page.Cell(5,2).SetFixedHeight(50, False)
	' R6
'	Dim labelLocoObstacleDistance As ABMLabel	'Set as class global to be able to update with the distance value set
	labelLocoObstacleDistance.Initialize(page,"labelLocoObstacleDistance",$"Obstacle Stop:{BR}${LocoObstacleDistance} cm"$, ABM.SIZE_PARAGRAPH, True, "bluegrey")
	page.Cell(6,1).AddComponent(labelLocoObstacleDistance)
	page.Cell(6,1).SetFixedHeight(50, False)
	Dim sliderLocoObstacleDistance As ABMSlider
	sliderLocoObstacleDistance.Initialize(page, "sliderLocoObstacleDistance", LocoObstacleDistance, 0, 20, 2, "themesliderlocoobstacledistance")
	sliderLocoObstacleDistance.HandleToolTip = "wNumb({ decimals: 0 })"
	sliderLocoObstacleDistance.Connect = ABM.SLIDER_CONNECT_LOWER
	page.Cell(6,2).AddComponent(sliderLocoObstacleDistance)
	page.Cell(6,2).SetFixedHeight(50, False)

	' NOT USED for now
	'page.AddExtraCSSFile("custom/lococontrol.css")
	
	' Floating Footer
	ABMShared.BuildFooter(page)
	
	' NOTE
	' Customization of CSS is done in sub ConnectPage using InjectCSS
End Sub

public Sub ConnectPage()			
	'	connecting the navigation bar
	ABMShared.ConnectNavigationBar(page)
	myToastId = myToastId + 1
	
	' Inject custom CSS for selectors. The selectors are defined in BuildPage and must be used in lowercase in InjectCSS
	' EXAMPLE: Button btnLocoBreak
'	page.InjectCSS($"#btnlocobreak {
'                   	background-color: red !important;
'                    }"$)
	' Slider Loco Speed Height
	page.InjectCSS($"#sliderlocospeed {
					height: 5px;
                    }
					#sliderlocoobstacledistance {
					height: 5px;
                    }"$)

	' refresh the page
	page.Refresh
	
	' Tell the browser we finished loading
	page.FinishedLoading
	' restoring the navigation bar position
	page.RestoreNavigationBarPosition	

End Sub

Sub page_MsgboxResult(returnName As String, result As String)
	
End Sub

Sub Page_Dropped(Params As Map)
	If Params.Get("before") = "" Then
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target"))
	Else
		Log("'" & Params.Get("component") & "' moved from " & Params.Get("source") & " to " & Params.Get("target") & " before component " & Params.Get("before"))
	End If	
End Sub

' clicked on the navigation bar
Sub Page_NavigationbarClicked(Action As String, Value As String)
	' saving the navigation bar position
	page.SaveNavigationBarPosition
	If Action = "ABMNavigationBar" Then Return
	If Action = "LogOff" Then
		ABMShared.LogOff(page)
		Return
	End If
	If Action = "About" Then
		'MessageBox
		page.Msgbox("MSGBOXABOUT", "LocoControl (c) 2017 by Robert W.B. Linn @{AL}http://www.rwblinn.de{AT}rwblinn.de{/AL}", "About", "OK", True, ABM.MSGBOX_POS_CENTER_CENTER, "bluegrey")
'		Toast
'		myToastId = myToastId + 1	
'		page.ShowToast("toast" & myToastId, "", "LocoControl (c) 2017 by Robert W.B. Linn", 5000, True)
		Return
	End If
	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub

Sub Page_InputboxResult(returnName As String, result As String)
	
End Sub

Sub Page_DebugConsole(message As String)
	Log("---> " & message)
End Sub

Sub Page_FileUploaded(FileName As String, success As Boolean)
	
End Sub

Sub Page_ToastClicked(toastID As String, Action As String)
		
End Sub

Sub Page_ToastDismissed(ToastId As String)
	
End Sub

Sub Page_Authenticated(Params As Map)
	
End Sub

Sub Page_FirebaseAuthError(extra As String)
	
End Sub

Sub Page_FirebaseAuthStateChanged(IsLoggedIn As Boolean)
	
End Sub

Sub Page_FirebaseStorageError(jobID As String, extra As String)
	
End Sub

Sub Page_FirebaseStorageResult(jobID As String, extra As String)
	
End Sub

Sub Page_ModalSheetDismissed(ModalSheetName As String)
	
End Sub

Sub Page_NextContent(TriggerComponent As String)
	
End Sub

Sub Page_SignedOffSocialNetwork(Network As String, Extra As String)
	
End Sub

Sub page_CellClicked(Target As String, RowCell As String)
	
End Sub

Sub page_RowClicked(Target As String, Row As String)
	
End Sub

Sub Page_NativeResponse(jsonMessage As String)
	
End Sub

Sub Page_NavigationbarSearch(search As String)
	
End Sub

#Region PageComponentAction
Sub btnLocoReverse_Clicked(Target As String)
	Log($"Clicked ${Target}"$)
	LocoDirection = "REV"
	labelLocoDirection.Text = LocoDirection
	Dim payload As String = LocoSpeed * 10
	Main.mqtt.Publish(Main.mqttTopicTrainReverse, payload.GetBytes("UTF8"))
	page.Refresh
End Sub

Sub btnLocoBreak_Clicked(Target As String)
	Log($"Clicked ${Target}"$)
	LocoDirection = "BRK"
	labelLocoDirection.Text = LocoDirection
	Dim payload As String = "0"
	Main.mqtt.Publish(Main.mqttTopicTrainBreak, payload.GetBytes("UTF8"))
	page.Refresh
End Sub

Sub btnLocoForwards_Clicked(Target As String)
	Log($"Clicked ${Target}"$)
	LocoDirection = "FWD"
	labelLocoDirection.Text = LocoDirection
	Dim payload As String = LocoSpeed * 10
	Main.mqtt.Publish(Main.mqttTopicTrainForwards, payload.GetBytes("UTF8"))
	page.Refresh
End Sub

Sub sliderLocoSpeed_Changed(value As String)
	Log("Speed new value: " & value)
	LocoSpeed = value
	Select LocoDirection
		Case "FWD"
			btnLocoForwards_Clicked("")
		Case "REV"
			btnLocoReverse_Clicked("")
	End Select
	labelLocoSpeed.Text = $"Speed set:{BR}${LocoSpeed} km/h"$
	page.Refresh
End Sub

Sub switchLocoHeadlight_Clicked(Target As String)
	Log($"Clicked ${Target}"$)
	Dim switch1 As ABMSwitch = page.Component("switchLocoHeadlight")
	Dim payload As String
	Log($"State ${switch1.State}"$)
	If switch1.State Then
		payload = 255
	Else
		payload = 0
	End If
	Main.mqtt.Publish(Main.mqttTopicTrainHeadlight, payload.GetBytes("UTF8"))
End Sub

Sub btnLocoBuzzer_Clicked(Target As String)
	Log($"Clicked ${Target}"$)
	Dim payload As String = "1"
	Main.mqtt.Publish(Main.mqttTopicTrainBuzzer, payload.GetBytes("UTF8"))
End Sub

Sub sliderLocoObstacleDistance_Changed(value As String)
	Log("LocoObstacleDistancenew value: " & value)
	labelLocoObstacleDistance.Text = $"Obstacle Stop:{BR}${value} cm"$
	Main.mqtt.Publish(Main.mqttTopicTrainDistanceThreshold, value.GetBytes("UTF8"))
	page.Refresh
End Sub

#End Region
